/* Generated file (by generator) */

/*
 * Copyright (C) 2015 William Yu <williamyu@gnome.org>
 *
 * This library is free software: you can redistribute it and/or modify it
 * under the terms of version 2.1. of the GNU Lesser General Public License
 * as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library. If not, see <https://www.gnu.org/licenses/>.
 */

#if !defined (__LIBICAL_GLIB_H_INSIDE__) && !defined (LIBICAL_GLIB_COMPILATION)
#error "Only <libical-glib/libical-glib.h> can be included directly."
#endif

#ifndef I_CAL_FORWARD_DECLARATIONS_H
#define I_CAL_FORWARD_DECLARATIONS_H

#include <glib-object.h>

G_BEGIN_DECLS

typedef struct _ICalArray ICalArray;
typedef struct _ICalAttach ICalAttach;
typedef struct _ICalCompIter ICalCompIter;
typedef struct _ICalComponent ICalComponent;
typedef struct _ICalDatetimeperiod ICalDatetimeperiod;
typedef struct _ICalDuration ICalDuration;
typedef struct _ICalGeo ICalGeo;
typedef struct _ICalParameter ICalParameter;
typedef struct _ICalParser ICalParser;
typedef struct _ICalPeriod ICalPeriod;
typedef struct _ICalProperty ICalProperty;
typedef struct _ICalRecurIterator ICalRecurIterator;
typedef struct _ICalRecurrence ICalRecurrence;
typedef struct _ICalReqstat ICalReqstat;
typedef struct _ICalTime ICalTime;
typedef struct _ICalTimeSpan ICalTimeSpan;
typedef struct _ICalTimezone ICalTimezone;
typedef struct _ICalTrigger ICalTrigger;
typedef struct _ICalValue ICalValue;

G_END_DECLS

#endif /* I_CAL_FORWARD_DECLARATIONS_H */
